@@ -17,9 +17,8 @@ from starlette.middleware.errors import ServerErrorMiddleware
 from starlette.middleware.cors import CORSMiddleware
 from starlette.middleware.gzip import GZipMiddleware
 from starlette.middleware.httpsredirect import HTTPSRedirectMiddleware
-from starlette.middleware.lifespan import LifespanMiddleware
 from starlette.middleware.trustedhost import TrustedHostMiddleware
-from starlette.routing import Router, LifespanHandler
+from starlette.routing import Lifespan
 from starlette.staticfiles import StaticFiles
 from starlette.testclient import TestClient
 from starlette.websockets import WebSocket
@@ -160,7 +159,7 @@ class API:
             self.add_route(self.docs_route, self.docs_response)
 
         self.default_endpoint = None
-        self.app = self.dispatch
+        self.app = self.asgi
         self.add_middleware(GZipMiddleware)
 
         if self.hsts_enabled:
@@ -168,13 +167,13 @@ class API:
 
         self.add_middleware(TrustedHostMiddleware, allowed_hosts=self.allowed_hosts)
 
-        self.lifespan_handler = LifespanMiddleware(LifespanHandler)
+        self.lifespan_handler = Lifespan()
 
         if self.cors:
             self.add_middleware(CORSMiddleware, **self.cors_params)
         self.add_middleware(ServerErrorMiddleware, debug=debug)
 
-        # Jinja enviroment
+        # Jinja environment
         self.jinja_env = jinja2.Environment(
             loader=jinja2.FileSystemLoader(
                 [str(self.templates_dir), str(self.built_in_templates_dir)],
@@ -254,10 +253,10 @@ class API:
     def add_middleware(self, middleware_cls, **middleware_config):
         self.app = middleware_cls(self.app, **middleware_config)
 
-    def __call__(self, scope):
-
+    async def __call__(self, scope, receive, send):
         if scope["type"] == "lifespan":
-            return self.lifespan_handler(scope)
+            await self.lifespan_handler(scope, receive, send)
+            return
 
         path = scope["path"]
         root_path = scope.get("root_path", "")
@@ -268,30 +267,49 @@ class API:
                 scope["path"] = path[len(path_prefix) :]
                 scope["root_path"] = root_path + path_prefix
                 try:
-                    return app(scope)
+                    await app(scope, receive, send)
+                    return
                 except TypeError:
                     app = WSGIMiddleware(app)
-                    return app(scope)
+                    await app(scope, receive, send)
+                    return
 
-        return self.app(scope)
+        await self.app(scope, receive, send)
 
-    def dispatch(self, scope):
-        # Call the main dispatcher.
-        async def asgi(receive, send):
-            nonlocal scope, self
+    async def asgi(self, scope, receive, send):
+        assert scope["type"] in ("http", "websocket")
 
-            if scope["type"] == "lifespan":
-                return self.lifespan_handler(scope)
-            elif scope["type"] == "websocket":
-                await self._dispatch_ws(scope=scope, receive=receive, send=send)
-            else:
-                req = models.Request(scope, receive=receive, api=self)
-                resp = await self._dispatch_request(
-                    req, scope=scope, send=send, receive=receive
-                )
-                await resp(receive, send)
+        if scope["type"] == "websocket":
+            await self._dispatch_ws(scope=scope, receive=receive, send=send)
+        else:
+            req = models.Request(scope, receive=receive, api=self)
+            resp = await self._dispatch_http(
+                req, scope=scope, send=send, receive=receive
+            )
+            await resp(scope, receive, send)
 
-        return asgi
+    async def _dispatch_http(self, req, **options):
+        # Set formats on Request object.
+        req.formats = self.formats
+
+        # Get the route.
+        route = self.path_matches_route(req.url.path)
+        route = self.routes.get(route)
+        if route:
+            resp = models.Response(req=req, formats=self.formats)
+
+            for before_request in self.before_http_requests:
+                await self.background(before_request, req=req, resp=resp)
+
+            await self._execute_route(route=route, req=req, resp=resp, **options)
+        else:
+            resp = models.Response(req=req, formats=self.formats)
+            self.default_response(req=req, resp=resp, notfound=True)
+        self.default_response(req=req, resp=resp)
+
+        self._prepare_session(resp)
+
+        return resp
 
     async def _dispatch_ws(self, scope, receive, send):
         ws = WebSocket(scope=scope, receive=receive, send=send)
@@ -357,29 +375,6 @@ class API:
     def no_response(req, resp, **params):
         pass
 
-    async def _dispatch_request(self, req, **options):
-        # Set formats on Request object.
-        req.formats = self.formats
-
-        # Get the route.
-        route = self.path_matches_route(req.url.path)
-        route = self.routes.get(route)
-        if route:
-            resp = models.Response(req=req, formats=self.formats)
-
-            for before_request in self.before_http_requests:
-                await self.background(before_request, req=req, resp=resp)
-
-            await self._execute_route(route=route, req=req, resp=resp, **options)
-        else:
-            resp = models.Response(req=req, formats=self.formats)
-            self.default_response(req=req, resp=resp, notfound=True)
-        self.default_response(req=req, resp=resp)
-
-        self._prepare_session(resp)
-
-        return resp
-
     async def _execute_route(self, *, route, req, resp, **options):
 
         params = route.incoming_matches(req.url.path)
@@ -550,7 +545,7 @@ class API:
 
     def on_event(self, event_type: str, **args):
         """Decorator for registering functions or coroutines to run at certain events
-        Supported events: startup, cleanup, shutdown, tick
+        Supported events: startup, shutdown
 
         Usage::
 
@@ -558,11 +553,7 @@ class API:
             async def open_database_connection_pool():
                 ...
 
-            @api.on_event('tick', seconds=10)
-            async def do_stuff():
-                ...
-
-            @api.on_event('cleanup')
+            @api.on_event('shutdown')
             async def close_database_connection_pool():
                 ...
 
@@ -647,15 +638,11 @@ class API:
 
         template = env.get_template("/".join([self.docs_theme, "index.html"]))
 
-        def static_url(asset):
-            assert None not in (self.static_dir, self.static_route)
-            return f"{self.static_route}/{asset}"
-
         return template.render(
             document=document,
             langs=["javascript", "python"],
             code_style=None,
-            static_url=static_url,
+            static_url=self.static_url,
             schema_url="/schema.yml",
         )
 
