@@ -14,7 +14,7 @@ import yaml
 from requests.structures import CaseInsensitiveDict
 from requests.cookies import RequestsCookieJar
 from starlette.datastructures import MutableHeaders
-from starlette.requests import Request as StarletteRequest
+from starlette.requests import Request as StarletteRequest, State
 from starlette.responses import (
     Response as StarletteResponse,
     StreamingResponse as StarletteStreamingResponse,
@@ -178,6 +178,19 @@ class Request:
         except AttributeError:
             return QueryDict({})
 
+    @property
+    def state(self) -> State:
+        """
+        Use the state to store additional information.
+        
+        This can be a very helpful feature, if you want to hand over
+        information from a middelware or a route decorator to the
+        actual route handler.
+
+        For example: ``request.state.time_started = time.time()`` 
+        """
+        return self._starlette.state
+
     @property
     async def encoding(self):
         """The encoding of the Request's body. Can be set, manually. Must be awaited."""
@@ -354,7 +367,7 @@ class Response:
         )
         starlette_response.raw_headers.extend(cookie_header)
 
-    async def __call__(self, receive, send):
+    async def __call__(self, scope, receive, send):
         body, headers = await self.body
         if self.headers:
             headers.update(self.headers)
@@ -367,4 +380,4 @@ class Response:
         response = response_cls(body, status_code=self.status_code, headers=headers)
         self._prepare_cookies(response)
 
-        await response(receive, send)
+        await response(scope, receive, send)
